/* Copyright (C) 2004 Cesar Garcia Tapia <tapia@eitig.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using System;
using System.Text;
using System.Collections;
using Gtk;
using Gdk;

public class BorrowerList: TreeView
{
	private Database database;
	private TreeStore store;
	
	private Catalog catalog;
	
	public BorrowerList (Catalog catalog, Database database)
	{
		this.database = database;
		
		this.catalog = catalog;
		catalog.OnBorrowerListChanged += new CollectionChangedHandler (Refresh);
		
		store = new TreeStore (typeof(string));
		this.Model = store;
		
		CellRendererPixbuf pr = new CellRendererPixbuf ();
		CellRendererText   tr = new CellRendererText ();
		TreeViewColumn column = new TreeViewColumn ();
		column.Title = Mono.Posix.Catalog.GetString ("Borrowers");
		column.PackStart (pr, false);
		column.SetCellDataFunc (pr, new TreeCellDataFunc (ImageCellDataFunc));
		column.PackStart (tr, false);
		column.AddAttribute (tr, "text", 1);
		column.SetCellDataFunc (tr, new TreeCellDataFunc (ItemCellDataFunc));
		this.AppendColumn (column);
		
		this.HeadersVisible = false;
		this.RulesHint = false;
		this.Visible = true;
		
		this.ShowAll();
		
		Fill ();
	}
	
	private void Fill ()
	{
		foreach (Borrower borrower in database.GetBorrowers()) {
			if (borrower.Items.Count > 0) {
				TreeIter iter = store.AppendValues(borrower.Name);
				foreach (int item in borrower.Items) {
					store.AppendValues (iter, item.ToString());
				}
			}
		}
	}
	
	private void Refresh ()
	{
		((TreeStore)this.Model).Clear();
		Fill();
	}
	
	private void ImageCellDataFunc(TreeViewColumn col,
			CellRenderer   cell,
			TreeModel      model,
			TreeIter       iter)
	{
		string itemId = (string)model.GetValue(iter, 0);

		CellRendererPixbuf cr = (CellRendererPixbuf) cell;
		try {
			Pixbuf pix = database.GetBorrowerItemPixbuf (Int32.Parse(itemId));
			cr.Pixbuf = pix;
		}
		catch {
			cr.Pixbuf = Pixbuf.LoadFromResource ("empty.png");
		}
	}

	private void ItemCellDataFunc(TreeViewColumn col,
			CellRenderer   cell,
			TreeModel      model,
			TreeIter       iter)
	{
		string itemId = (string)model.GetValue(iter, 0);
		
		try {
			string s = database.GetBorrowerItemText (Int32.Parse(itemId));
			((CellRendererText)cell).Text = s;
		}
		catch {
			StringBuilder sb = new StringBuilder (itemId);
			sb = sb.Append ("(");
			sb = sb.Append (model.IterNChildren(iter));
			sb = sb.Append (")");
			((CellRendererText)cell).Text = sb.ToString();
		}
	}
}
