/* Copyright (C) 2004 Cesar Garcia Tapia <tapia@eitig.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using System;
using System.Collections;
using Gtk;
using Gnome;

public class CatalogList: TreeView
{
	private event EventHandler OnItemSelectionChanged;
	private event EventHandler OnAddItemRequest;
	private event EventHandler OnLendItemRequest;
	private event EventHandler OnReturnItemRequest;
	private event EventHandler OnEditItemRequest;
	private event EventHandler OnDeleteItemRequest;
	private ScrolledWindow swItems;
	private ScrolledWindow swPresentation;
	private ItemList activeItemList;
	private ItemShelf activeItemShelf;
	private int count;
	private View view;

	private Database database;
	private Catalog catalog;

	public CatalogList (ScrolledWindow swItems, ScrolledWindow swPresentation, Database database)
	{
		this.swItems = swItems;
		this.swPresentation = swPresentation;
		this.count = 0;

		this.database = database;

		CellRendererPixbuf pr = new CellRendererPixbuf ();
		CellRendererText   tr = new CellRendererText ();
		TreeViewColumn column = new TreeViewColumn ();
		column.Title = Mono.Posix.Catalog.GetString ("Catalogs");
		column.PackStart (pr, false);
		column.SetCellDataFunc (pr, new TreeCellDataFunc (ImageCellDataFunc));
		column.PackStart (tr, false);
		column.AddAttribute (tr, "text", 1);
		column.SetCellDataFunc (tr, new TreeCellDataFunc (CatalogCellDataFunc));
		this.AppendColumn (column);
		this.Model = new ListStore (typeof(Catalog));
		this.HeadersVisible = false;
		this.RulesHint = false;
		this.Visible = true;
		this.Selection.Changed += new EventHandler (SelectionChanged);

		this.PopupMenu += new Gtk.PopupMenuHandler (OnPopupMenu);
		this.ButtonReleaseEvent += new Gtk.ButtonReleaseEventHandler (OnButtonReleased);
	}

	public void AddCatalog (Catalog catalog)
	{
		((ListStore)this.Model).AppendValues(catalog);
		count++;
	}

	public Catalog GetSelectedCatalog ()
	{
		TreeIter      iter;
		TreeModel     model;
		Catalog       catalog;

		if (!this.Selection.GetSelected(out model, out iter)) {
			return null;
		}

		return (Catalog)model.GetValue(iter, 0);
	}

	public void SetSelectedCatalog (int catalog)
	{
		this.Selection.SelectPath (new TreePath (catalog.ToString()));
	}

	public int Count ()
	{
		return count;
	}

	public ItemList ActiveItemList
	{
		get {
			return activeItemList;
		}
	}

	public ItemShelf ActiveItemShelf
	{
		get {
			return activeItemShelf;
		}
	}

	public View ActiveView
	{
		get {
			return this.view;
		}
		set {
			this.view = value;
			SelectionChanged (null, null);
		}
	}

	public void Search (string searchString)
	{
		GetSelectedCatalog().ItemCollection.Search (searchString);
	}

	private void SelectionChanged (object o, EventArgs args)
	{
		if (this.Selection.CountSelectedRows() > 0) {
			this.catalog = GetSelectedCatalog ();
			catalog.ItemCollection.LoadAll();
			if (activeItemList == null || activeItemList.Catalog != catalog) {
				Console.WriteLine ("CREANDO ITEMLIST");
				activeItemList = database.LoadItemList(catalog);
				activeItemList.PresentationWidget = swPresentation;
				activeItemList.OnItemSelected      += new EventHandler (ItemSelectionChanged);
				activeItemList.OnLendItemRequest   += new EventHandler (LendItemRequest);
				activeItemList.OnReturnItemRequest += new EventHandler (ReturnItemRequest);
				activeItemList.OnEditItemRequest   += new EventHandler (EditItemRequest);
				activeItemList.OnItemDeleted       += new EventHandler (DeleteItemRequest);
			}

			if (activeItemShelf == null || activeItemShelf.Catalog != catalog) {
				Console.WriteLine ("CREANDO SHELF");
				activeItemShelf = database.LoadItemShelf (catalog);
				activeItemShelf.PresentationWidget = swPresentation;
				activeItemShelf.OnItemSelected      += new EventHandler (ItemSelectionChanged);
				activeItemShelf.OnLendItemRequest   += new EventHandler (LendItemRequest);
				activeItemShelf.OnReturnItemRequest += new EventHandler (ReturnItemRequest);
				activeItemShelf.OnEditItemRequest   += new EventHandler (EditItemRequest);
				activeItemShelf.OnItemDeleted       += new EventHandler (DeleteItemRequest);
			}

			if (swItems.Child != null) {
				swItems.Remove (swItems.Child);
				if (swPresentation.Child != null) {
					swPresentation.Remove (swPresentation.Child);
				}
			}

			switch (view) {
				case View.List:
					swItems.Child = activeItemList;
					break;
				case View.Shelf:
					swItems.Child = activeItemShelf;
					swItems.SizeAllocated += new SizeAllocatedHandler (activeItemShelf.SizeAllocate);
					break;
			}
			swItems.ResizeChildren();
		}
	}

	private void ItemSelectionChanged (object o, EventArgs args)
	{
		if (OnItemSelectionChanged != null) {
			OnItemSelectionChanged (null, null);
		}
	}

	private void DeleteItemRequest (object o, EventArgs args)
	{
		if (this.OnDeleteItemRequest != null) {
			this.OnDeleteItemRequest (null, null);
		}
	}

	private void AddItemRequest (object o, EventArgs args)
	{
		if (this.OnAddItemRequest != null) {
			this.OnAddItemRequest (null, null);
		}
	}
	
	private void LendItemRequest (object o, EventArgs args)
	{
		if (this.OnLendItemRequest != null) {
			this.OnLendItemRequest (null, null);
		}
	}
	
	private void ReturnItemRequest (object o, EventArgs args)
	{
		if (this.OnReturnItemRequest != null) {
			this.OnReturnItemRequest (null, null);
		}
	}

	private void EditItemRequest (object o, EventArgs args)
	{
		if (this.OnEditItemRequest != null) {
			this.OnEditItemRequest (null, null);
		}
	}

	private void OnPopupMenu (object o, Gtk.PopupMenuArgs args)
	{
		if (this.Selection.CountSelectedRows() > 0) {
			ShowPopup ();
		}
	}

	private void OnButtonReleased (object o, Gtk.ButtonReleaseEventArgs args)
	{
		if (this.Selection.CountSelectedRows() > 0) {
			if (args.Event.Button == 3)
				ShowPopup ();
		}
	}

	private void ShowPopup ()
	{
		Gtk.Menu menu = new Gtk.Menu ();
		menu.AccelGroup = new AccelGroup ();
		Gtk.ImageMenuItem properties = new Gtk.ImageMenuItem (Gtk.Stock.Properties, menu.AccelGroup);
		properties.Activated += new EventHandler (OnPropertiesMenuItemClicked);

		menu.Append (properties);

		menu.Popup (null, null, null, IntPtr.Zero, 3, Gtk.Global.CurrentEventTime);
		menu.ShowAll ();
	}

	private void OnPropertiesMenuItemClicked (object o, EventArgs args)
	{
		CatalogPropertiesDialog dialog = new CatalogPropertiesDialog (GetSelectedCatalog());
		dialog.Run();
		SelectionChanged (null, null);
	}

	private void ImageCellDataFunc(TreeViewColumn col,
			CellRenderer   cell,
			TreeModel      model,
			TreeIter       iter)
	{
		Catalog catalog = (Catalog)model.GetValue(iter, 0);

		CellRendererPixbuf cr = (CellRendererPixbuf) cell;
		cr.Pixbuf = catalog.Image.Pixbuf;
	}

	private void CatalogCellDataFunc(TreeViewColumn col,
			CellRenderer   cell,
			TreeModel      model,
			TreeIter       iter)
	{
		Catalog catalog = (Catalog)model.GetValue(iter, 0);
		((CellRendererText)cell).Text = catalog.ShortDescription;
	}
}
