/*
 * Copyright (C) 2004 Cesar Garcia Tapia <tapia@eitig.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using System;
using System.Text;
using System.Collections;
using System.IO;

using Gtk;
using Gdk;

public class Item
{
	private ArrayList columns;
	private string cover;
	
	private Gdk.Pixbuf coverPixbuf;
	private Gdk.Pixbuf coverSmallPixbuf;
	private Gdk.Pixbuf coverMediumPixbuf;
	private Gdk.Pixbuf coverLargePixbuf;
	
	private string table;
	private bool selected;

	public Item (string table, ArrayList columns)
	{
		this.columns = columns;
		this.table = table;
		
		if (columns[1] != null) {
			cover = columns[1].ToString();
		}
		else {
			cover = null;
		}

		if (cover != null) {
			try {
				coverPixbuf  = new Gdk.Pixbuf (cover);
			}
			catch {
				coverPixbuf = null;
			}
		}
		else {
			coverPixbuf = null;
		}
		
		cover = SetCache (ImageSize.Large, cover);
		columns[1] = cover;
		SetCache (ImageSize.Medium, cover);
		SetCache (ImageSize.Small, cover);
		
		coverLargePixbuf = GetLargePixbuf ();
		coverMediumPixbuf = GetMediumPixbuf ();
		coverSmallPixbuf = GetSmallPixbuf ();
	}

	private Gdk.Pixbuf CoverPixbuf
	{
		get {
			return coverPixbuf;
		}
	}

	public bool IsSelected {
		get {
			return selected;
		}
		set {
			selected = value;
		}
	}
	
	public ArrayList Columns {
		get {
			return columns;
		}
	}

	public int Id {
		get {
			if (columns[0] != null) {
				return Int32.Parse (columns[0].ToString());
			}
			else {
				return -1;
			}
		}
	}

	public string Title {
		get {
			if (columns[3] != null) {
				return columns[3].ToString();
			}
			else {
				return "";
			}
		}
	}
	
	public string Table {
		get {
			return this.table;
		}
	}

	public string ImagePath {
		get {
			return cover;
		}
	}

	private Gdk.Pixbuf GetSmallPixbuf () {
		Gdk.Pixbuf pixbuf;
		if (coverPixbuf != null) {
			pixbuf = GetCache (ImageSize.Small);
			if (pixbuf == null) {
				if (this.CoverPixbuf.Height > this.CoverPixbuf.Width) {
					int x = 50*this.CoverPixbuf.Width/this.CoverPixbuf.Height;
					pixbuf = this.CoverPixbuf.ScaleSimple (x, 50, InterpType.Hyper);
				}
				else {
					int x = 50*this.CoverPixbuf.Height/this.CoverPixbuf.Width;
					pixbuf = this.CoverPixbuf.ScaleSimple (50, x, InterpType.Hyper);
				}
				SetCache (ImageSize.Small, cover);
			}
			return pixbuf;
		}
		else {
			return null;
		}
	}

	private Gdk.Pixbuf GetMediumPixbuf () {
		Gdk.Pixbuf pixbuf;
		if (coverPixbuf != null) {
			pixbuf = GetCache (ImageSize.Medium);
			if (pixbuf == null) {
				if (this.CoverPixbuf.Height > this.CoverPixbuf.Width) {
					int x = 100*this.CoverPixbuf.Width/this.CoverPixbuf.Height;
					pixbuf = this.CoverPixbuf.ScaleSimple (x, 100, InterpType.Hyper);
				}
				else {
					int x = 100*this.CoverPixbuf.Height/this.CoverPixbuf.Width;
					pixbuf = this.CoverPixbuf.ScaleSimple (100, x, InterpType.Hyper);
				}
				SetCache (ImageSize.Medium, cover);
			}
			return pixbuf;
		}
		else {
			return null;
		}
	}

	private Gdk.Pixbuf GetLargePixbuf () {
		Gdk.Pixbuf pixbuf;
		if (coverPixbuf != null) {
			pixbuf = this.CoverPixbuf;
			return pixbuf;
		}
		else {
			return null;
		}
	}

	public Gdk.Pixbuf SmallCover {
		get {
			return coverSmallPixbuf; 
		}
	}

	public Gdk.Pixbuf MediumCover {
		get {
			return coverMediumPixbuf;
		}
	}

	public Gdk.Pixbuf LargeCover {
		get {
			return coverLargePixbuf;
		}
	}

	public FileStream GetCoverStream (ImageSize size)
	{
		string file;
		
		if (Conf.Get("ui/preview_cache", true)) {
			StringBuilder cacheDir = new StringBuilder (Conf.HomeDir);
			cacheDir = cacheDir.Append ("/cache");
			switch (size) {
				case ImageSize.Small:
					cacheDir = cacheDir.Append ("/small/");
					cacheDir = cacheDir.Append (table);
					cacheDir = cacheDir.Append (Id);
					cacheDir = cacheDir.Append (".png");
					file= cacheDir.ToString();
					if (File.Exists (file)) {
						return File.OpenRead (file);
					}
					break;
				case ImageSize.Medium:
					cacheDir = cacheDir.Append ("/medium/");
					cacheDir = cacheDir.Append (table);
					cacheDir = cacheDir.Append (Id);
					cacheDir = cacheDir.Append (".png");
					file= cacheDir.ToString();
					if (File.Exists (file)) {
						return File.OpenRead (file);
					}
					break;
				case ImageSize.Large:
					cacheDir = cacheDir.Append ("/large/");
					cacheDir = cacheDir.Append (table);
					cacheDir = cacheDir.Append (Id);
					cacheDir = cacheDir.Append (".png");
					file = cacheDir.ToString();
					if (File.Exists (file)) {
						return File.OpenRead (file);
					}
					break;
			}
		}
		return File.OpenRead (cover);
	}

	private Gdk.Pixbuf GetCache (ImageSize size)
	{
		string file;

		if (Conf.Get("ui/preview_cache", true)) {
			StringBuilder cacheDir = new StringBuilder (Conf.HomeDir);
			cacheDir = cacheDir.Append ("/cache");
			switch (size) {
				case ImageSize.Small:
					cacheDir = cacheDir.Append ("/small/");
					cacheDir = cacheDir.Append (table);
					cacheDir = cacheDir.Append (Id);
					cacheDir = cacheDir.Append (".png");
					file = cacheDir.ToString();
					if (File.Exists (file)) {
						return new Gdk.Pixbuf (file);
					}
					else {
						return null;
					}
				case ImageSize.Medium:
					cacheDir = cacheDir.Append ("/medium/");
					cacheDir = cacheDir.Append (table);
					cacheDir = cacheDir.Append (Id);
					cacheDir = cacheDir.Append (".png");
					file = cacheDir.ToString();
					if (File.Exists (file)) {
						return new Gdk.Pixbuf (file);
					}
					else {
						return null;
					}
				case ImageSize.Large:
					cacheDir = cacheDir.Append ("/large/");
					cacheDir = cacheDir.Append (table);
					cacheDir = cacheDir.Append (Id);
					cacheDir = cacheDir.Append (".png");
					file = cacheDir.ToString();
					if (File.Exists (file)) {
						return new Gdk.Pixbuf (file);
					}
					else {
						return null;
					}
				default:
					return null;
			}
		}
		else {
			return null;
		}
	}

	private string SetCache (ImageSize size, string file)
	{
		string fileAux = null;
		StringBuilder cacheDir;
		Gdk.Pixbuf pixbuf;
		
		if (Conf.Get("ui/preview_cache", true)) {
			switch (size) {
				case ImageSize.Small:
					if (this.CoverPixbuf.Height > this.CoverPixbuf.Width) {
						int x = 50*this.CoverPixbuf.Width/this.CoverPixbuf.Height;
						pixbuf = this.CoverPixbuf.ScaleSimple (x, 50, InterpType.Hyper);
					}
					else {
						int x = 50*this.CoverPixbuf.Height/this.CoverPixbuf.Width;
						pixbuf = this.CoverPixbuf.ScaleSimple (50, x, InterpType.Hyper);
					}
					cacheDir = new StringBuilder (Conf.HomeDir);
					cacheDir = cacheDir.Append ("/cache");
					cacheDir = cacheDir.Append ("/small/");
					cacheDir = cacheDir.Append (table);
					cacheDir = cacheDir.Append (Id);
					cacheDir = cacheDir.Append (".png");
					fileAux = cacheDir.ToString();
					pixbuf.Save (fileAux, "png");
					break;
				case ImageSize.Medium:
					if (this.CoverPixbuf.Height > this.CoverPixbuf.Width) {
						int x = 100*this.CoverPixbuf.Width/this.CoverPixbuf.Height;
						pixbuf = this.CoverPixbuf.ScaleSimple (x, 100, InterpType.Hyper);
					}
					else {
						int x = 100*this.CoverPixbuf.Height/this.CoverPixbuf.Width;
						pixbuf = this.CoverPixbuf.ScaleSimple (100, x, InterpType.Hyper);
					}
					cacheDir = new StringBuilder (Conf.HomeDir);
					cacheDir = cacheDir.Append ("/cache");
					cacheDir = cacheDir.Append ("/medium/");
					cacheDir = cacheDir.Append (table);
					cacheDir = cacheDir.Append (Id);
					cacheDir = cacheDir.Append (".png");
					fileAux = cacheDir.ToString();
					pixbuf.Save (fileAux, "png");
					
					break;
				case ImageSize.Large:
					cacheDir = new StringBuilder (Conf.HomeDir);
					cacheDir = cacheDir.Append ("/cache");
					cacheDir = cacheDir.Append ("/large/");
					cacheDir = cacheDir.Append (table);
					cacheDir = cacheDir.Append (Id);
					cacheDir = cacheDir.Append (".png");
					fileAux = cacheDir.ToString();
					this.CoverPixbuf.Save (fileAux, "png");
					break;
			}		
		}

		return fileAux;
	}

	public static void RemoveCache (string table, int id)
	{
		try {
			StringBuilder cacheDir = new StringBuilder (300);
			cacheDir = cacheDir.Append (Conf.HomeDir);
			
			StringBuilder file = new StringBuilder (100);
			file = file.Append (table);
			file = file.Append (id);
			file = file.Append (".png");

			StringBuilder aux;
			
			aux = cacheDir.Append ("/cache/small/");
			aux = aux.Append (file.ToString());
			File.Delete (aux.ToString());

			aux = cacheDir.Append ("/cache/medium/");
			aux = aux.Append (file.ToString());
			File.Delete (aux.ToString());

			aux = cacheDir.Append ("/cache/large/");
			aux = aux.Append (file.ToString());
			File.Delete (aux.ToString());
		}
		catch {}
	}
}
