/*
 * Copyright (C) 2004 Cesar Garcia Tapia <tapia@eitig.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using System;
using System.Collections;

using Gtk;
using Gdk;
using Gnome;

public class ItemShelf: Gnome.Canvas
{
	public event EventHandler OnEditItemRequest;
	public event EventHandler OnLendItemRequest;
	public event EventHandler OnReturnItemRequest;
	public event EventHandler OnItemSelected;
	public event EventHandler OnItemDeleted;
	
	private double zoom = 1;
	private static int DEFAULT_ELEMENT_SIZE = 110;
	private int ELEMENT_SIZE = 110;

	private int verticalPos = 10;
	private int horizontalPos = 10;

	private ScrolledWindow swPresentation;
	
	private Catalog catalog;
	private ArrayList elements;
	
	private ItemShelfElement activeElement;
	
	private Pixbuf background;
	private ArrayList backgrounds;
	private Pixbuf shelf;
	private ArrayList shelfs;
	
	private Tooltips tips;

	public ItemShelf (Catalog catalog): base ()
	{
		this.Visible = true;
		this.elements = new ArrayList();
		this.catalog = catalog;
		this.tips = new Tooltips();

		string themePath = Defines.IMAGE_DATADIR + "/" + Conf.Get ("ui/theme", "default");
		background = new Pixbuf (themePath + "/background.jpg");
		shelf = new Pixbuf (themePath + "/shelf.png");
	 
		catalog.OnItemCollectionChanged += new CollectionChangedHandler (Refresh);

		LoadAll ();
	}

	public void ChangeTheme (string theme)
	{
		Conf.Set("ui/theme", theme);
		string themePath = Defines.IMAGE_DATADIR + "/" + Conf.Get ("ui/theme", "default");
		background = new Pixbuf (themePath + "/background.jpg");
		shelf = new Pixbuf (themePath + "/shelf.png");
		Resize();
	}
	
	public Catalog Catalog {
		get {
			return catalog;
		}
	}
		
	private void LoadAll ()
	{
		foreach (Item item in catalog.ItemCollection) {
			this.AddItem (item);
		}
	}

	private void Resize ()
	{	
		Gdk.Rectangle rect = this.Allocation;
		int height;
		if (rect.Height > verticalPos + ELEMENT_SIZE + 10) {
			height = rect.Height;
		}
		else {
			height = verticalPos + ELEMENT_SIZE + 10;
		}
		this.SetScrollRegion (0.0, 0.0, rect.Width, height);
		this.SetBackground (rect.Width, height);
		this.DrawShelf (rect.Width, height);
	}
	
	private void SetBackground (int width, int height)
	{
		if (backgrounds == null) {
			backgrounds = new ArrayList ();
		}
		else {
			CanvasPixbuf cp;
			for (int c = 0; c < backgrounds.Count; c++) {
				cp = (CanvasPixbuf)backgrounds[c];
				cp.Destroy();
			}
			backgrounds.Clear();
		}
	
		int bgX = 0;
		int bgY = 0;
		
		if (bgX == 0 && background.Height >= this.Allocation.Height) {
			return;
		}
		
		while (true) {
			Gnome.CanvasPixbuf cp = new Gnome.CanvasPixbuf (this.Root());
			cp.Pixbuf = background;
			cp.LowerToBottom();
			cp.X = bgX;
			cp.Y = bgY;
			backgrounds.Add (cp);
						
			if (bgX + background.Width >= width) {
				bgX = 0;
				bgY += background.Height;
			}
			else {	
				bgX += background.Width;
			}
			
			if (bgX == 0 && bgY > height) {
				return;
			}	
		}
	}
	
	private void DrawShelf (int width, int height)
	{
		if (shelfs == null) {
			shelfs = new ArrayList ();
		}
		else {
			CanvasPixbuf cp;
			for (int c = 0; c < shelfs.Count; c++) {
				cp = (CanvasPixbuf)shelfs[c];
				cp.Destroy();
			}
			shelfs.Clear();
		}
	
		int shelfX = 0;
		int shelfY = ELEMENT_SIZE + 10;
		
		if (shelfX == 0 && shelf.Height >= this.Allocation.Height) {
			return;
		}
		
		while (true) {
			Gnome.CanvasPixbuf cp = new Gnome.CanvasPixbuf (this.Root());
			cp.Pixbuf = shelf;
			cp.RaiseToTop();
			cp.X = shelfX;
			cp.Y = shelfY;
			shelfs.Add (cp);
						
			if (shelfX + shelf.Width >= width) {
				shelfX = 0;
				shelfY += ELEMENT_SIZE + 10;
			}
			else {	
				shelfX += shelf.Width;
			}
			
			if (shelfX == 0 && shelfY > height) {
				return;
			}	
		}
	}
	
	public double Zoom {
		get {
			return zoom;
		}
		set {
			if (value != zoom) {
				zoom = value;
				ELEMENT_SIZE = (int)(DEFAULT_ELEMENT_SIZE * zoom);
				Resize ();
				Refresh ();
			}
		}
	}
	
	private void AddItem (Item item)
	{
		ItemShelfElement element = new ItemShelfElement (this.Root(), item, ELEMENT_SIZE);
		if (catalog.IsBorrowed (item)) {
			element.IsBorrowed = true;
		}
						
		element.Data["item"] = item;
		element.CanvasEvent += new Gnome.CanvasEventHandler (OnElementClicked);
		elements.Add (element);
	}

	private void DrawAll ()
	{    
		horizontalPos = 10;
		verticalPos = 10;
		foreach (ItemShelfElement element in elements) {
			DrawWidget (element);
		}
	}
	
	private void DrawWidget (ItemShelfElement element)
	{
		element.SetPos (horizontalPos, verticalPos);
		
		if (horizontalPos + ELEMENT_SIZE*2 + 20 >= this.Allocation.Width) {
			horizontalPos = 10;
			verticalPos += ELEMENT_SIZE + 10;
		}
		else {	
			horizontalPos += ELEMENT_SIZE + 10;
		}
	}

	private void Refresh () {
		Clear();
		LoadAll ();
		DrawAll ();
	}

	public void SizeAllocate (object o, SizeAllocatedArgs args)
	{
		Resize ();
		DrawAll ();
	}

	
	private void Clear ()
	{
		foreach (ItemShelfElement element in elements) {
			element.Destroy();
		}
		elements.Clear();
	}

	private void DeleteSelected ()
	{
		ArrayList toDelete = new ArrayList();
		foreach (ItemShelfElement element in elements) {
			if (element.Selected) {
				OnItemDeleted (null, null);
			}
		}
		Refresh();
	}

	public ScrolledWindow PresentationWidget
	{
		get {
			return this.swPresentation;
		}
		set {
			this.swPresentation = value;
		}
	}

	private void OnElementClicked (object o, Gnome.CanvasEventArgs args)
	{
		activeElement = (ItemShelfElement)o;
		Gdk.EventButton ev = new Gdk.EventButton (args.Event.Handle);
		
		if (args.Event.Type == Gdk.EventType.ButtonRelease) {
			
			foreach (ItemShelfElement element in elements) {
				if (element != activeElement) {
					element.Unselect();
				}
			}
						
			if (ev.State == (ModifierType.ControlMask|ModifierType.Button1Mask) && activeElement.Selected) {
				activeElement.Unselect();
			}
			else {
				activeElement.Select();
			}
			OnItemSelected (activeElement.Item, null);
			
			if (swPresentation.Child != null) {
				swPresentation.Remove (swPresentation.Child);
			}
			
			string template = "template_"+catalog.Table+".html";
			Presentation presentation = new Presentation (template, activeElement.Item);
			swPresentation.Child = presentation;
			swPresentation.ShowAll();
			
			if (ev.Button == 3) {
				ShowPopup ((ItemShelfElement)o);
			}
		}
		else if (args.Event.Type == Gdk.EventType.EnterNotify) {
			((ItemShelfElement)o).Highlight();
		}
		else if (args.Event.Type == Gdk.EventType.LeaveNotify) {
			((ItemShelfElement)o).UnHighlight();
		}
		else if (args.Event.Type == Gdk.EventType.TwoButtonPress) {
			OnEditItemRequest (null, null);
		}
	}

	private void ShowPopup (ItemShelfElement element)
	{
		Gtk.Menu menu = new Gtk.Menu ();
		menu.AccelGroup = new AccelGroup ();
		Gtk.MenuItem editMenuItem = new Gtk.MenuItem (Mono.Posix.Catalog.GetString ("Edit"));
		
		Gtk.MenuItem lendMenuItem;
		if (catalog.IsBorrowed (element.Item)) {
			lendMenuItem = new Gtk.MenuItem (Mono.Posix.Catalog.GetString ("Return"));
			lendMenuItem.Activated  += new EventHandler (OnReturnMenuItemClicked);
		}
		else {
			 lendMenuItem = new Gtk.MenuItem (Mono.Posix.Catalog.GetString ("Lend"));
			 lendMenuItem.Activated  += new EventHandler (OnLendMenuItemClicked);
		}
		
		Gtk.ImageMenuItem removeMenuItem = new Gtk.ImageMenuItem (Gtk.Stock.Remove, menu.AccelGroup);
		Gtk.SeparatorMenuItem separator = new Gtk.SeparatorMenuItem();

		editMenuItem.Activated += new EventHandler (OnEditMenuItemClicked);
		removeMenuItem.Activated += new EventHandler (OnRemoveMenuItemClicked);

		menu.Append (editMenuItem);
		menu.Append (lendMenuItem);
		menu.Append (separator);
		menu.Append (removeMenuItem);

		menu.Popup (null, null, null, IntPtr.Zero, 3, Gtk.Global.CurrentEventTime);	
		
		menu.ShowAll ();
	}
	
	private void OnEditMenuItemClicked (object o, EventArgs args)
	{
		OnEditItemRequest (null, null);
	}
	
	private void OnLendMenuItemClicked (object o, EventArgs args)
	{
		OnLendItemRequest(null, null);
	}
	
	private void OnReturnMenuItemClicked (object o, EventArgs args)
	{
		OnReturnItemRequest (null, null);
	}
	
	private void OnRemoveMenuItemClicked (object o, EventArgs args)
	{
		OnItemDeleted (null, null);
	}
}

public class ItemShelfElement: Gnome.CanvasGroup
{
	private CanvasPixbuf pixbufBackground;
	private CanvasPixbuf pixbuf;
	private CanvasPixbuf pixbufBorrowed;
	private CanvasText text;
	private CanvasRect rect;
	private ArrayList elements;
	
	public Item Item;
	private Pixbuf cover;
	private Pixbuf background;
	private Pixbuf borrowed;
	
	private int ELEMENT_SIZE;
	
	public ItemShelfElement (CanvasGroup group, Item item, int ELEMENT_SIZE): base (group, CanvasGroup.GType)
	{
		this.Item = item;
		this.ELEMENT_SIZE = ELEMENT_SIZE;
		
		if (item.LargeCover != null) {
			cover = item.LargeCover;
		}
		else {
			cover = new Gdk.Pixbuf (null, "shelfbgnp.png");
		}
		
		if (cover.Width > cover.Height) {
			cover = cover.ScaleSimple (ELEMENT_SIZE - 10, (cover.Height*(ELEMENT_SIZE - 10)/cover.Width), InterpType.Hyper);
		}
		else {
			cover = cover.ScaleSimple ((cover.Width*(ELEMENT_SIZE - 10)/cover.Height), ELEMENT_SIZE - 10, InterpType.Hyper);
		}

		this.pixbufBackground = new Gnome.CanvasPixbuf (this);
		background = new Gdk.Pixbuf (null, "shelfbg.png");
		background = background.ScaleSimple (cover.Width+10, cover.Height+10, InterpType.Hyper);
		
		this.pixbufBackground.Pixbuf = background;
		
		this.pixbuf = new Gnome.CanvasPixbuf (this);
		this.pixbuf.Pixbuf = cover;

		this.rect = new CanvasRect (this);
		this.rect.OutlineColor = "#0000ff";
		this.rect.WidthPixels = 2;			

		if (item.MediumCover == null) {		
			this.text = new Gnome.CanvasText (this);
			this.text.Text = item.Title;
			this.text.Font = "Sans Bold 10";
			this.text.FillColor = "black";
		}
		
		borrowed = null;
		this.Unselect();
	}
	
	public bool IsBorrowed {
		get {
			return borrowed != null;
		}
		set {
			if (value) {
				this.pixbufBorrowed = new Gnome.CanvasPixbuf (this);
				borrowed = Pixbuf.LoadFromResource ("borrowed.png");
				this.pixbufBorrowed.Pixbuf = borrowed;
			}
			else {
				this.pixbufBorrowed = null;
				borrowed = null;
			}
		}
	}
	
	public bool Selected {
		get {
			return this.Item.IsSelected;
		}
	}
	
	public void SetPos (int x, int y)
	{		
		this.pixbufBackground.X = x + (ELEMENT_SIZE / 2 - pixbufBackground.Pixbuf.Width / 2);
		this.pixbufBackground.Y = y + (ELEMENT_SIZE - pixbufBackground.Pixbuf.Height) + 5;

		this.pixbuf.X = x + (ELEMENT_SIZE / 2 - pixbuf.Pixbuf.Width / 2);
		this.pixbuf.Y = y + (ELEMENT_SIZE - pixbuf.Pixbuf.Height);

		if (this.text != null) {
			this.text.X = x + ELEMENT_SIZE / 2;
			this.text.Y = y + ELEMENT_SIZE / 2;
		}

		this.rect.X1 = x + (ELEMENT_SIZE / 2 - pixbuf.Pixbuf.Width / 2) - 2;
		this.rect.Y1 = y + (ELEMENT_SIZE - pixbuf.Pixbuf.Height) - 2;
		this.rect.X2 = this.rect.X1 + pixbuf.Pixbuf.Width + 2;
		this.rect.Y2 = this.rect.Y1 + pixbuf.Pixbuf.Height + 2;

		if (this.pixbufBorrowed != null) {
			this.pixbufBorrowed.X = x + ELEMENT_SIZE - pixbufBorrowed.Pixbuf.Width;
			this.pixbufBorrowed.Y = y + ELEMENT_SIZE - pixbufBorrowed.Pixbuf.Height + 1;
			this.pixbufBorrowed.RaiseToTop();
		}
	}
	
	public void Highlight ()
	{
		Pixbuf aux = (Pixbuf)cover.Clone ();
		this.pixbuf.Pixbuf.SaturateAndPixelate (aux, (float)2.0, false);
		this.pixbuf.Pixbuf = aux;
	}
	
	public void UnHighlight ()
	{
		this.pixbuf.Pixbuf = cover;
	}
	
	public void Select ()
	{
		this.rect.Show();
		this.Item.IsSelected = true;
	}
	
	public void Unselect ()
	{
		this.rect.Hide();
		this.Item.IsSelected = false;
	}
}
