/*
 * Copyright (C) 2004 Cesar Garcia Tapia <tapia@eitig.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using System;
using System.IO;
using System.Reflection;
using System.Collections;
using System.Text;
using System.Text.RegularExpressions;

using Gtk;

public class Presentation: Gtk.HTML
{
	private string template;
	private Item item;

	public Presentation (string template, Item item)
	{
		this.template = template;
		this.item = item;

		string htmlString = CreateHtmlString();

		this.UrlRequested += new UrlRequestedHandler (OnUrlRequested);
		HTMLStream stream = this.Begin ("text/html; charset=utf-8");
		stream.Write(htmlString);
		this.End (stream, HTMLStreamStatus.Ok);
	}

	private string CreateHtmlString ()
	{
		string htmlString = null;

		Assembly thisAssembly = Assembly.GetEntryAssembly ();
		Stream stream = thisAssembly.GetManifestResourceStream(template);
		if (stream != null) {
			StreamReader sr = new StreamReader (stream);
			template = sr.ReadToEnd();
			htmlString = FillTemplate (template);
		}
		else {
			htmlString = "<html><body><h1>ERROR:</h1><br><h3>No existe la plantilla</h3></body><html>";
		}

		return htmlString;
	}

	private string FillTemplate (string template)
	{
		string result = template;
		string pattern = @"##\w+##";

		foreach (Match m in Regex.Matches(template, pattern))
		{
			StringBuilder match = new StringBuilder (m.ToString());
			match.Replace ("##", "");
			string s = match.ToString();
			int col = Int32.Parse(s);
			
			if (item.Columns[col] != null) {
				string escaped = Regex.Escape (item.Columns[col].ToString());
				StringBuilder content = new StringBuilder (escaped);
				content.Replace ("\n", "<br>");
			
				result = Regex.Replace (result, m.ToString(), content.ToString());
			}

		}
		
		return result;
	}

	private void OnUrlRequested (object o, UrlRequestedArgs args)
	{
		try {
			FileStream s = item.GetCoverStream (ImageSize.Small);
			byte [] buffer = new byte [8192];
			int n;

			while ((n = s.Read (buffer, 0, 8192)) != 0) {
				args.Handle.Write (buffer, n);
			}
			args.Handle.Close (HTMLStreamStatus.Ok);
		} catch {
		}
	}
}
