/*
 * Copyright (C) 2004 Cesar Garcia Tapia <tapia@eitig.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using System;
using System.IO;
using System.Net;
using System.Threading;
using System.Collections;

using Gtk;
using Gnome;

using Amazon;

public class AddBookDialog: Dialog
{
	private Catalog catalog;
	private ArrayList values;
	
	private Gtk.Entry entryTitle;
	private Gtk.Entry entryOriginalTitle;
	private Gtk.Entry entryAuthor;
	private Gtk.Entry entryDate;
	private Gtk.Entry entryPages;
	private Gtk.Entry entryGenre;
	private Gtk.Entry entryISBN;
	private Gtk.Entry entryPublisher;
	private Gtk.Entry entryCountry;
	private Gtk.Entry entryLanguage;
	private Gnome.PixmapEntry entryImage;
	private Gtk.TextView textviewComments;
	private Gtk.HBox ratingBox;
	private RatingWidget ratingWidget;
	
	private int Update = -1;
	
	private ProgressDialog progressDialog;
	private Details details;
	private Thread thread;

	public AddBookDialog (Catalog catalog) : base ()
	{
		Glade.XML gxml = new Glade.XML (null, "dialogaddbook.glade", "vbox1", null);
			
		this.catalog = catalog;
		this.Title = Mono.Posix.Catalog.GetString ("Add a new book");
		this.HasSeparator = false;
		this.values = new ArrayList();

		Button cancelButton = (Button)this.AddButton (Gtk.Stock.Cancel, 0);
		Button okButton     = (Button)this.AddButton (Gtk.Stock.Ok, 1);
		cancelButton.Clicked += new EventHandler (OnCancelButtonClicked);
		okButton.Clicked     += new EventHandler (OnOkButtonClicked);

		VBox vBox = this.VBox;
		vBox.Add ((VBox)gxml["vbox1"]);
		
		entryTitle = (Gtk.Entry)gxml["entryTitle"];
		entryTitle.IsFocus = true;
		entryOriginalTitle = (Gtk.Entry)gxml["entryOriginalTitle"];
		entryAuthor = (Gtk.Entry)gxml["entryAuthor"];
		entryDate = (Gtk.Entry)gxml["entryDate"];
		entryPages = (Gtk.Entry)gxml["entryPages"];
		entryGenre = (Gtk.Entry)gxml["entryGenre"];
		entryISBN = (Gtk.Entry)gxml["entryIsbn"];
		entryPublisher = (Gtk.Entry)gxml["entryPublisher"];
		entryCountry = (Gtk.Entry)gxml["entryCountry"];
		entryLanguage = (Gtk.Entry)gxml["entryLanguage"];
		entryImage = (Gnome.PixmapEntry)gxml["pixmapentry"];
		textviewComments = (Gtk.TextView)gxml["textview1"];
		ratingBox = (Gtk.HBox)gxml["hboxRating"];
		
		ratingWidget = new RatingWidget();
		ratingWidget.Visible = true;
		ratingBox.Add (ratingWidget);
		
		Button buttonSearchAmazon = (Button)gxml["buttonSearchAmazon"];
		buttonSearchAmazon.Clicked += new EventHandler (OnSearchAmazonClicked);
		
		this.ShowAll();
	}
	
	public void EditItem (Item item)
	{
		this.Title = Mono.Posix.Catalog.GetString ("Editing");
		this.Update = item.Id;
		
		entryTitle.Text = item.Columns[3].ToString();
		entryOriginalTitle.Text = item.Columns[4].ToString();
		entryAuthor.Text = item.Columns[5].ToString();
		entryDate.Text = item.Columns[6].ToString();
		entryGenre.Text = item.Columns[7].ToString(); 
		entryPages.Text = item.Columns[8].ToString();
		entryPublisher.Text = item.Columns[9].ToString();
		entryISBN.Text = item.Columns[10].ToString();
		entryCountry.Text = item.Columns[11].ToString();
		entryLanguage.Text = item.Columns[12].ToString();
		entryImage.Filename = item.Columns[1]!=null?item.Columns[1].ToString():"";
		textviewComments.Buffer.Text = item.Columns[13].ToString();
		ratingWidget.Value = Int32.Parse (item.Columns[2].ToString());
	}
	
	private void FillDialogFromAmazon (Details details, string image)
	{
		entryTitle.Text = details.ProductName;
		
		entryAuthor.Text = "";
		for (int i = 0; i < details.Authors.Length; i++) {
			entryAuthor.Text += details.Authors[i];
			if (i+1 < details.Authors.Length) {
				entryAuthor.Text += ", ";
			}
		}
		
		entryDate.Text = details.ReleaseDate;
		entryPages.Text = details.NumberOfPages;
		entryISBN.Text = details.Isbn;
		entryPublisher.Text = details.Manufacturer;
		entryImage.Filename = image;
		if (details.ProductDescription != null) {
			textviewComments.Buffer.Text = details.ProductDescription;
		}  
	}
	
	public void OnCancelButtonClicked (object o, EventArgs args)
	{
		this.Destroy();
	}

	public void OnOkButtonClicked (object o, EventArgs args)
	{
		ArrayList columns = new ArrayList ();
		if (this.Update == -1) { 
			columns.Add (null);
		}
		else {
			columns.Add (this.Update);
		}
		
		if (entryImage.Filename != null) {
			FileInfo fileInfo = new FileInfo (entryImage.Filename);
			if (fileInfo.Length == 807) {
				columns.Add (null);
			}
			else {
				columns.Add (entryImage.Filename);
			}
		}
		else {
			columns.Add (null);
		}
		
		columns.Add (ratingWidget.Value.ToString());
		columns.Add (entryTitle.Text);
		columns.Add (entryOriginalTitle.Text);
		columns.Add (entryAuthor.Text);
		columns.Add (entryDate.Text);
		columns.Add (entryGenre.Text);
		columns.Add (entryPages.Text);
		columns.Add (entryPublisher.Text);
		columns.Add (entryISBN.Text);
		columns.Add (entryCountry.Text);
		columns.Add (entryLanguage.Text);
		columns.Add (textviewComments.Buffer.Text);
		
		if (this.Update == -1) {
			catalog.NewItem (new Item (catalog.Table, columns));
		}
		else {
			catalog.UpdateItem (new Item (catalog.Table, columns));
		}
		
		this.Destroy();
	}
	
	private void OnSearchAmazonClicked (object o, EventArgs args)
	{
		progressDialog = new ProgressDialog();
		progressDialog.ShowAll();
        thread = new Thread(new ThreadStart(amazonQuery));
        GLib.Idle.Add (new GLib.IdleHandler (Pulse));
        thread.Start(); 
	}
	
	private void amazonQuery ()
	{
		AmazonQuery amazonQuery = new AmazonQuery ();
		details = amazonQuery.Query ("books", entryTitle.Text);
		if (details != null) {
			FillDialogFromAmazon (details, amazonQuery.GetImage(details.ImageUrlMedium));
		}
		progressDialog.Destroy();
	}
	
	private bool Pulse ()
	{
		if (thread.IsAlive) {
			progressDialog.Pulse();
			Thread.Sleep (100);
			return true;
		}
		progressDialog.Destroy();
		return false;
	}
}
