/*
 * Copyright (C) 2004 Cesar Garcia Tapia <tapia@eitig.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using System;
using System.IO;
using System.Net;
using System.Threading;
using System.Collections;

using Gtk;
using Gnome;

using Amazon;

public class AddFilmDialog: Dialog
{
	private Catalog catalog;
	private ArrayList values;
	
	private Gtk.Entry entryTitle;
	private Gtk.Entry entryOriginalTitle;
	private Gtk.Entry entryDirector;
	private Gtk.Entry entryDate;
	private Gtk.Entry entryRuntime;
	private Gtk.Entry entryGenre;
	private Gtk.Entry entryDistributor;
	private Gtk.Entry entryMedium;
	private Gtk.Entry entryCountry;
	private Gtk.Entry entryLanguage;
	private Gnome.PixmapEntry entryImage;
	private Gtk.TextView textviewComments;
	private Gtk.TextView textviewStarring;
	private Gtk.HBox ratingBox;
	private RatingWidget ratingWidget;
	
	private int Update = -1;
	
	private ProgressDialog progressDialog;
	private Details details;
	
	private Thread thread;

	public AddFilmDialog (Catalog catalog) : base ()
	{
		Glade.XML gxml = new Glade.XML (null, "dialogaddfilm.glade", "vbox1", null);
			
		this.catalog = catalog;
		this.Title = Mono.Posix.Catalog.GetString ("Add a new film");
		this.HasSeparator = false;
		this.values = new ArrayList();

		Button cancelButton = (Button)this.AddButton (Gtk.Stock.Cancel, 0);
		Button okButton     = (Button)this.AddButton (Gtk.Stock.Ok, 1);
		cancelButton.Clicked += new EventHandler (OnCancelButtonClicked);
		okButton.Clicked     += new EventHandler (OnOkButtonClicked);

		VBox vBox = this.VBox;
		vBox.Add ((VBox)gxml["vbox1"]);
		
		entryTitle = (Gtk.Entry)gxml["entryTitle"];
		entryTitle.IsFocus = true;
		entryOriginalTitle = (Gtk.Entry)gxml["entryOriginalTitle"];
		entryDirector = (Gtk.Entry)gxml["entryDirector"];
		entryDate = (Gtk.Entry)gxml["entryDate"];
		entryRuntime = (Gtk.Entry)gxml["entryRuntime"];
		entryGenre = (Gtk.Entry)gxml["entryGenre"];
		entryDistributor = (Gtk.Entry)gxml["entryDistributor"];
		entryMedium = (Gtk.Entry)gxml["entryMedium"];
		entryCountry = (Gtk.Entry)gxml["entryCountry"];
		entryLanguage = (Gtk.Entry)gxml["entryLanguage"];
		entryImage = (Gnome.PixmapEntry)gxml["pixmapentry"];
		textviewComments = (Gtk.TextView)gxml["textview1"];
		textviewStarring = (Gtk.TextView)gxml["textviewStarring"];
		ratingBox = (Gtk.HBox)gxml["hboxRating"];
				
		ratingWidget = new RatingWidget();
		ratingWidget.Visible = true;
		ratingBox.Add (ratingWidget);
		
		Button buttonSearchAmazon = (Button)gxml["buttonSearchAmazon"];
		buttonSearchAmazon.Clicked += new EventHandler (OnSearchAmazonClicked);
		
		this.ShowAll();
	}
	
	public void EditItem (Item item)
	{
		this.Title = Mono.Posix.Catalog.GetString ("Edit");
		this.Update = item.Id;
		
		entryImage.Filename = item.Columns[1]!=null?item.Columns[1].ToString():"";
		ratingWidget.Value = Int32.Parse (item.Columns[2].ToString());
		entryTitle.Text = item.Columns[3].ToString();
		entryOriginalTitle.Text = item.Columns[4].ToString();
		entryDirector.Text = item.Columns[5].ToString();
		textviewStarring.Buffer.Text = item.Columns[6].ToString();
		entryDate.Text = item.Columns[7].ToString();
		entryGenre.Text = item.Columns[8].ToString();
		entryRuntime.Text = item.Columns[9].ToString();
		entryCountry.Text = item.Columns[10].ToString();
		entryLanguage.Text = item.Columns[11].ToString();
		entryDistributor.Text = item.Columns[12].ToString();
		entryMedium.Text = item.Columns[13].ToString();
		textviewComments.Buffer.Text = item.Columns[14].ToString();
	}
	
	public void FillDialogFromAmazon (Details details, string image)
	{
		entryImage.Filename = image;
		
		entryTitle.Text = details.ProductName;
		
		entryDirector.Text = "";
		if (details.Directors != null) {
			for (int i = 0; i < details.Directors.Length; i++) {
				if (i > 0) entryDirector.Text += ", ";
				entryDirector.Text += details.Directors[i];
			}
		}
		
		if (details.TheatricalReleaseDate != null) {
			entryDate.Text = details.TheatricalReleaseDate.Substring (0, 4);
		}
		
		entryGenre.Text = "";
		entryRuntime.Text = details.RunningTime;
		entryCountry.Text = ""; 
		entryLanguage.Text = "";
		entryDistributor.Text = details.Manufacturer;
		entryMedium.Text = "DVD";
		
		if (details.ProductDescription != null) {
			textviewComments.Buffer.Text = details.ProductDescription;
		}
		
		textviewStarring.Buffer.Text = "";
		if (details.Starring != null) {
			for (int i = 0; i < details.Starring.Length; i++) {
				textviewStarring.Buffer.Text += details.Starring[i];
				if (i+1 < details.Starring.Length) {
					textviewStarring.Buffer.Text += "\n";
				}
			}
		}
	}
	
	public void OnCancelButtonClicked (object o, EventArgs args)
	{
		this.Destroy();
	}

	public void OnOkButtonClicked (object o, EventArgs args)
	{
		ArrayList columns = new ArrayList ();
		if (this.Update == -1) { 
			columns.Add (null);
		}
		else {
			columns.Add (this.Update);
		}
		
		if (entryImage.Filename != null) {
			FileInfo fileInfo = new FileInfo (entryImage.Filename);
			if (fileInfo.Length == 807) {
				columns.Add (null);
			}
			else {
				columns.Add (entryImage.Filename);
			}
		}
		else {
			columns.Add (null);
		}
		
		columns.Add (ratingWidget.Value.ToString());
		columns.Add (entryTitle.Text);
		columns.Add (entryOriginalTitle.Text);
		columns.Add (entryDirector.Text);
		columns.Add (textviewStarring.Buffer.Text);
		columns.Add (entryDate.Text);
		columns.Add (entryGenre.Text);
		columns.Add (entryRuntime.Text);
		columns.Add (entryCountry.Text);
		columns.Add (entryLanguage.Text);
		columns.Add (entryDistributor.Text);
		columns.Add (entryMedium.Text);
		columns.Add (textviewComments.Buffer.Text);
		
		if (this.Update == -1) {
			catalog.NewItem (new Item (catalog.Table, columns));
		}
		else {
			catalog.UpdateItem (new Item (catalog.Table, columns));
		}
		
		this.Destroy();
	}
	
	private void OnSearchAmazonClicked (object o, EventArgs args)
	{
		progressDialog = new ProgressDialog();
		progressDialog.ShowAll();
		thread = new Thread(new ThreadStart(amazonQuery));
		GLib.Idle.Add (new GLib.IdleHandler (Pulse));
		thread.Start(); 
	}
	
	private void amazonQuery ()
	{
		AmazonQuery amazonQuery = new AmazonQuery ();
		details = amazonQuery.Query ("dvd", entryTitle.Text);
		if (details != null) {
			FillDialogFromAmazon (details, amazonQuery.GetImage (details.ImageUrlMedium));
		}
		progressDialog.Destroy();
	}
	
	private bool Pulse ()
	{
		if (thread.IsAlive) {
			progressDialog.Pulse();
			Thread.Sleep (100);
			return true;
		}
		progressDialog.Destroy();
		return false;
	}
}
