using System;
using System.Collections;

using Gtk;
using Gdk;

public class LendItemDialog: Dialog
{
	private Database database;
	private Item item;
	
	private Gtk.Image image;
	private Label label;
	private Gtk.Combo entry;
	
	private ArrayList borrowers;
	
	public LendItemDialog (Item item, Database database)
	{
		this.database = database;
		this.item = item;
		this.Title = Mono.Posix.Catalog.GetString ("Lend");
		this.Modal = true;
		this.HasSeparator = false;
		
		VBox vBox = this.VBox;
		HBox hBox1 = new HBox ();
		
		Pixbuf pixbuf = item.LargeCover;
		if (pixbuf != null) {
			image = new Gtk.Image (pixbuf);
		}
		
		label = new Label ();
		// Escape specials Markups characters
		string title;
		title = item.Title.Replace ("&", "&amp;");
		title = title.Replace ("<", "&lt;");
		title = title.Replace (">", "&gt;");
		label.Text = "<b>"+title+"</b>";
		label.UseMarkup = true;
		
		Label label1 = new Label (Mono.Posix.Catalog.GetString ("Borrower:"));
		entry = new Gtk.Combo ();
		entry.ValueInList = false;
		entry.AllowEmpty = false;
		
		int i=0;
		borrowers = database.GetBorrowers ();
		string [] names = new string[borrowers.Count];
		foreach (Borrower borrower in borrowers) {
			names[i++] = borrower.Name;
		} 
		entry.PopdownStrings = names;
		
		hBox1.PackStart (label1);
		hBox1.PackStart (entry);
		
		Gtk.Frame frame = new Frame ();
		frame.Shadow = ShadowType.Out;
		frame.Add (image);
		
		vBox.PackStart (frame);
		vBox.PackStart (label);
		vBox.PackStart (hBox1);
		vBox.Spacing = 5;
		
		Button cancelButton = (Button)this.AddButton (Gtk.Stock.Cancel, 0);
		Button okButton = (Button)this.AddButton (Gtk.Stock.Ok, 1);
		
		cancelButton.Clicked += new EventHandler (OnCancelButtonClicked);
		okButton.Clicked     += new EventHandler (OnOkButtonClicked);
		
		this.ShowAll();
	}
	
	public void OnOkButtonClicked (object o, EventArgs args)
	{
		Borrower lendTo = null;
		foreach (Borrower borrower in borrowers) {
			if (borrower.Name.Equals (entry.Entry.Text)) {
				lendTo = borrower;
				break;
			}
		}
		
		if (lendTo == null) {
			int id = database.AddBorrower (entry.Entry.Text);
			lendTo = new Borrower (id, entry.Entry.Text, null);
		}
		
		database.LendItem (item, lendTo);
		
		this.Destroy();
	}
	
	public void OnCancelButtonClicked (object o, EventArgs args)
	{
		this.Destroy();
	}
}
